﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace ReportSerializerApp
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }
        
        private void MainForm_Load(object sender, EventArgs e)
        {
            reportInfoBindingSource.DataSource = ReportManager.GetReports();
        }

        private void btnSaveReport_Click(object sender, EventArgs e)
        {
            using (SaveFileDialog dlg = new SaveFileDialog())
            {
                dlg.Filter = "xml files (*.xml)|*.xml|All files (*.*)|*.*";

                if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    SerializeReport(dlg.FileName);
                }
            }
        }

        private void SerializeReport(string filename)
        {
            StartProgressBar();

            // create the report
            Type reportType = (Type)lstReports.SelectedValue;
            object report = Activator.CreateInstance(reportType);

            ThreadPool.QueueUserWorkItem(w =>
            {
                // serialize the report
                using (System.Xml.XmlWriter xmlWriter = System.Xml.XmlWriter.Create(filename))
                {
                    System.Xml.Serialization.XmlSerializer xmlSerializer = new System.Xml.Serialization.XmlSerializer(typeof(Telerik.Reporting.Report));
                    xmlSerializer.Serialize(xmlWriter, report);
                }

                StopProgressBar();
            });
        }

        private void StartProgressBar()
        {
            if (progressBar1.InvokeRequired)
                progressBar1.Invoke(new MethodInvoker(this.StartProgressBar));

            btnSaveReport.Enabled = false;
            progressBar1.Style = ProgressBarStyle.Marquee;
            progressBar1.MarqueeAnimationSpeed = 30;
        }

        private void StopProgressBar()
        {
            if (progressBar1.InvokeRequired)
                progressBar1.Invoke(new MethodInvoker(this.StopProgressBar));

            btnSaveReport.Enabled = true;
            progressBar1.Style = ProgressBarStyle.Continuous;
            progressBar1.MarqueeAnimationSpeed = 0;
        }

        #region Properties

        Telerik.Reporting.Examples.CSharp.ReportManager _reportManager;
        public Telerik.Reporting.Examples.CSharp.ReportManager ReportManager
        {
            get
            {
                if (_reportManager == null)
                    _reportManager = new Telerik.Reporting.Examples.CSharp.ReportManager();

                return _reportManager;
            }
        }

        #endregion
    }
}
